﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;

namespace CKPoolSet
{
    /// <summary>
    /// DataTable扩展功能
    /// </summary>
    public static class TableExtensions
    {
        /// <summary>
        /// 根据传入的匿名实体将DataTable转为List形式，如果是实体，可以自己设置实体状态
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="dataTable"></param>
        /// <returns></returns>
        public static List<T> ToList<T>(this DataTable dataTable) where T : class
        {
            return DoRead<T>(dataTable);
        }


        /// <summary>
        /// 将IEnumerable转化为DataTable，为了防止命名冲突而增加V2
        /// </summary>
        /// <param name="enumerable"></param>
        /// <returns></returns>
        public static DataTable ToDataTable<T>(this IEnumerable<T> enumerable)
        {
            var _dataTable = new DataTable();
            var _properties = TypeDescriptor.GetProperties(typeof(T));
            foreach (PropertyDescriptor pd in _properties)
            {
                _dataTable.Columns.Add(pd.Name, pd.PropertyType);
            }
            foreach (T item in enumerable)
            {
                var _row = _dataTable.NewRow();
                var _rowItemArray = _row.ItemArray;
                int _index = 0;
                foreach (PropertyDescriptor pd in _properties)
                {
                    _rowItemArray[_index] = pd.GetValue(item);
                    _index++;
                }
                _row.ItemArray = _rowItemArray;
                _dataTable.Rows.Add(_row);
            }
            foreach (DataColumn _col in _dataTable.Columns)
            {
                foreach (PropertyDescriptor pd in _properties)
                {
                    if (_col.ColumnName.ToUpper() == pd.Name.ToUpper())
                    {
                        _col.ExtendedProperties.Add(typeof(T).ToString(), pd.Name);
                        break;
                    }
                }
            }
            _dataTable.AcceptChanges();
            _dataTable.RemotingFormat = SerializationFormat.Binary;
            return _dataTable;
        }


        private static string GetEntityId(Type type)
        {
            return type.Name.Replace("Entity", "") + "Id";
        }

        /// <summary>
        /// 从DataTable到List
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="dataTable"></param>
        /// <returns></returns>
        private static List<T> DoRead<T>(DataTable dataTable)
        {
            Dictionary<string, PropertyInfo> _convertProperties = new Dictionary<string, PropertyInfo>();
            PropertyInfo[] _propertyInfos = typeof(T).GetProperties();
            foreach (var item in _propertyInfos)
            {
                if (!IsBaseType(item.PropertyType))
                    continue;
                foreach (DataColumn _column in dataTable.Columns)
                {
                    if (item.Name.ToUpper() == _column.ColumnName.ToUpper())
                    {
                        _convertProperties.Add(_column.ColumnName.ToUpper(), item);
                        break;
                    }
                }
            }
            List<T> _entities = new List<T>();
            for (int i = 0; i < dataTable.Rows.Count; i++)
            {
                T _entity = Activator.CreateInstance<T>();
                for (int j = 0; j < dataTable.Columns.Count; j++)
                {
                    string _columnName = dataTable.Columns[j].ColumnName;
                    if (string.IsNullOrEmpty(_columnName)) continue;
                    _columnName = _columnName.ToUpper();
                    var _dRow = dataTable.Rows[i];
                    if (_convertProperties.ContainsKey(_columnName))
                    {
                        if (_dRow[j] is System.DBNull)
                        {
                            //如果是int类型,不允许出现dbnull,将dbnumm转为0
                            if (dataTable.Columns[j].DataType == typeof(int))
                                _dRow[j] = 0;
                            else
                            {
                                _dRow[j] = DefaultForType(_convertProperties[_columnName].PropertyType);
                            }
                        }
                        if (_dRow[j] == DBNull.Value) continue;
                        if (IsBaseType(
                            _convertProperties[_columnName].PropertyType))
                        {
                            try
                            {
                                _convertProperties[_columnName].SetValue(_entity, Convert.ChangeType(_dRow[j], _convertProperties[_columnName].PropertyType), null);
                            }
                            catch
                            {
                                continue;
                            }

                        }
                    }
                }
                _entities.Add(_entity);
            }

            return _entities;
        }

        /// <summary>
        /// 是否是数据库要处理的属性
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static bool IsBaseType(Type type)
        {
            bool result = false;
            if (StandardTypes.Contains(type.FullName.ToLower()))
                result = true;
            return result;
        }

        /// <summary>
        /// 判断类型(或其基类)与类型全名是否相同
        /// </summary>
        /// <param name="type"></param>
        /// <param name="typeFullName"></param>
        /// <returns></returns>
        internal static bool IsEqualType(Type type, string typeFullName)
        {
            bool _result = false;
            while (type != null)
            {
                if (type.FullName == typeFullName)
                {
                    _result = true; break;
                }
                type = type.BaseType;
            }
            return _result;
        }

        private static List<string> _standardTypes = new List<string>();

        /// <summary>
        /// 可使用于映射的标准类型
        /// </summary>
        private static List<string> StandardTypes
        {
            get
            {
                if (_standardTypes.Count == 0)
                {
                    _standardTypes.Add("system.string");
                    _standardTypes.Add("system.int32");
                    _standardTypes.Add("system.boolean");
                    _standardTypes.Add("system.double");
                    _standardTypes.Add("system.single");
                    _standardTypes.Add("system.datetime");
                    _standardTypes.Add("system.byte[]");
                    _standardTypes.Add("system.byte");
                    _standardTypes.Add("system.decimal");
                    _standardTypes.Add("system.guid");
                    _standardTypes.Add("system.int16");
                    _standardTypes.Add("system.int64");
                    _standardTypes.Add("system.sbyte");
                    _standardTypes.Add("system.decimal");
                    _standardTypes.Add("system.timespan");
                    _standardTypes.Add("system.uint16");
                    _standardTypes.Add("system.uint32");
                    _standardTypes.Add("system.uint64");
                }
                return _standardTypes;
            }
            //set { _standardTypes = value; }
        }

        private static object DefaultForType(Type targetType)
        {
            if (targetType == typeof(string)) return "";
            return targetType.IsValueType ? Activator.CreateInstance(targetType) : null;
        }
    }
}
